# A. For application in unlisted dataframe (or each element of a list of df)----

calcGapDuration <- function(hourIn, HourOutOfPrevious, units = 'secs'){
  difftime(hourIn, dplyr::lag(HourOutOfPrevious), units=units)  
}

get_pairs <- function(df,threshold){
  output <- df %>% 
    mutate(displacer = id,
           displaced = lag(id)
    ) %>%
    filter(gap_duration < threshold)
  
  return(output)
}

graph_by_list_element <- function(df, isdirected){
  thegraph <- df %>% select(displacer, displaced) %>% 
    graph_from_data_frame(., directed = isdirected)
  return(thegraph)  
}

gapdurationlist <- function(thelist) {
  mylist <- thelist
  
  for (item in 1:length(mylist)) {
    unlisted <- mylist[[item]] 
    unlisted$gap_duration <- calcGapDuration(unlisted$hour.in, unlisted$hour.out)
    mylist[[item]] <- unlisted
  }
  return(mylist)
}

createConsData <- function(df, Mode) {
  consumption.ind.daily <- df %>%
    ungroup() %>%
    filter(id != "FILLING") %>%
    group_by(Date, id) %>%
    summarise(
      daily.cons.ind = sum(cons),
      daily.bouts = n(),
      station = unique(station)
    ) %>%
    mutate(avg.intake.per.bout = daily.cons.ind / daily.bouts) %>%
    ungroup() %>% group_by(station)
  
  # per station
  consumption.station.daily <- consumption.ind.daily %>%
    ungroup() %>%
    group_by(station, Date) %>%
    summarise(daily.cons.group = sum(daily.cons.ind), n = length(unique(id))) %>%
    mutate(mean.daily.cons = daily.cons.group/ n)
  
  if (Mode == "perstation") {
    return(consumption.station.daily)
  }
  
  if (Mode == "all") {
    # Adding the group consumption to the individual
    consumption.all <-
      left_join(consumption.ind.daily,
                consumption.station.daily,
                by = c('station', 'Date')) %>%
      mutate(proportion = daily.cons.ind / daily.cons.group) %>%
      arrange(station)
    return(consumption.all)
  }
}

# B. For application in lists of dataframes ----

get_pairs_list <- function(thelist, mythreshold) {
  
  for (item in 1:length(thelist)) {
    temp <- thelist[[item]]
    temp <- get_pairs(temp,threshold = mythreshold)
    thelist[[item]] <- temp
  }
  return(thelist)
}

createdegreeslist <- function(thelist) {
  mylist <- thelist
  
  for (item in 1:length(mylist)) {
    unlisted <- createdegrees(mylist[[item]])
    mylist[[item]] <- unlisted
  }
  return(mylist)
}


savetheplot_pliz <- function(thepath, plot, station){
  ggsave(file = paste0(thepath, "Edge density - Station ",
                       unique(dataset_raw$station)[station], 
                       ".png"),
         plot = plot,
         device = "png")
}

makeIGraphObjects <- function(pairDF, directed) {
  # outer loop: stations
  for (station in 1:length(pairDF)) {
    # inner loop: dates
    #message("This is station", station)
    
    # the -1 in the length is because there is a weird last element for each station
    
    for (date in 1:(length(pairDF[[station]])-1)) {
      #message("This is date", date)
      temp <- pairDF[[station]][[date]]
      temp <- graph_by_list_element(temp, isdirected = directed)
      pairDF[[station]][[date]] <- temp
    }
  }
  return(pairDF)
}

makePairsPerStation <- function(df, mythreshold) {
  
  # outer loop: stations
  for (station in 1:length(df)) {
    # inner loop: dates
    #message("This is station", station)
    # the -1 in the length is because there is a weird last element for each station
    
    for (date in 1:(length(df[[station]])-1)) {
      #message("This is date", date)
      temp <- df[[station]][[date]]
      temp <- mergevisits(temp)
      temp <- get_pairs(temp, threshold = mythreshold)
      df[[station]][[date]] <- temp
    }
  }
  return(df)
}

makeIGraphObjects <- function(pairDF, directed) {
  # outer loop: stations
  for (station in 1:length(pairDF)) {
    # inner loop: dates
    #message("This is station", station)
    
    # the -1 in the length is because there is a weird last element for each station
    
    for (date in 1:(length(pairDF[[station]])-1)) {
      #message("This is date", date)
      temp <- pairDF[[station]][[date]]
      temp <- graph_by_list_element(temp, isdirected = directed)
      pairDF[[station]][[date]] <- temp
    }
  }
  return(pairDF)
}

# TODO: solve artifact when pigs visit feeder across midnight

visitPlotsDay <- function(df, thestation, thedate, singlestrip) {
  thedata <- df %>% 
    filter(Date == ymd(thedate)) %>%
    filter(station == thestation)
  
  if(singlestrip == TRUE){
    thedata <- mutate(thedata, visit = 0)
    
    ggplot(transform(thedata, y=order(id, hour.in)), aes(
      x    = hour.in,
      xend = hour.out,
      y    = visit,
      yend = visit,
      col  = id
    )) + geom_segment(size=20)+
      scale_y_discrete(breaks=NULL)
  } else {
    ggplot(transform(thedata, y=order(id, hour.in)), aes(
      x    = hour.in,
      xend = hour.out,
      y    = id,
      yend = id,
      col  = id
    )) + geom_segment(size=5)+
      scale_y_discrete(breaks=NULL)
  }
}

inspectDay <- function(df, thestation){
  s_x <- df[df$station == thestation,] # filtrando station x
  s_x$s_time <- hour(s_x$hour.in) + minute(s_x$hour.in)/60
  
  p1 <- ggplot(s_x, aes(x = Date, y = s_time))+ 
    geom_point(alpha = 0.5) +geom_jitter()
  
  p2 <- populationPlot(s_x)
  
  grid.arrange(p1, p2, nrow = 2)
}

# C. For application in a graph object ----

createdegrees <- function(thegraph) {
  vector_length <- length(V(thegraph))
  
  if(vector_length > 0) {
    V(thegraph)$label <- V(thegraph)$name
    V(thegraph)$degree <- degree(thegraph)}
  
  return(thegraph)
}

makeEdgeDensities <- function(df_net) {
  # outer loop: stations
  for (station in 1:length(df_net)) {
    # inner loop: dates
    #message("This is station", station)
    
    # the -1 in the length is because there is a weird last element for each station
    
    for (date in 1:(length(df_net[[station]])-1)) {
      #message("This is date", date)
      temp <- df_net[[station]][[date]]
      temp <- edge_density(temp)
      df_net[[station]][[date]] <- temp
    }
    
  }
  return(unlist2d(df_net))
}



#used in Redemption site
savetheplot_pliz.2 <- function(thepath, myplot, station){
  ggsave(file = paste0(thepath, "Edge density - Station ",
                       station, 
                       ".png"),
         plot = myplot,
         device = "png")
}

# D. Super functions to rule them all ----

### Getmetheplot, a quick method to output degree density
getmetheplot_pliz <- function(site, df, thestation) {
  df_s_n <- df[df$station == thestation,] %>%
    arrange(Date, hour.in)
  dataset_perday <- df_s_n %>% 
    ungroup() %>% # remove station grouping
    group_by(Date) %>% 
    mutate(date_ymd = Date) %>%
    group_map(~ mutate(.x, 
                       gap_duration = calcGapDuration(hour.in, hour.out))) %>%
    setNames(unique(sort(df_s_n$Date))) # this line preserves station number!
  
  alldates <- names(dataset_perday)
  
  dataset_perday.merged <- lapply(dataset_perday, function(i) mergevisits(i)) 
  dataset_perday.merged.gd <- gapdurationlist(dataset_perday.merged)
  
  pairvisits_list <- get_pairs_list(dataset_perday.merged.gd, mythreshold = 20)
  graph_list <- lapply(pairvisits_list, function(i) graph_by_list_element(i,T))
  ggg_simple <- lapply(graph_list, function(i) simplify(i))
  ggg_simple <- createdegreeslist(ggg_simple)
  
  ed_bydate <- sapply(ggg_simple, function(i) edge_density(i)) %>% 
    rbind(names(dataset_perday)) %>% 
    as.data.frame() %>%
    transpose() %>%
    rename(date = V2,
           edgeDensity = V1) 
  
  plot <- ggplot(ed_bydate, aes(x = as.Date(date), y = as.numeric(edgeDensity))) + geom_point() + 
    scale_x_date(date_breaks = "2 weeks") + 
    ylab("Edge density") + 
    xlab("Date") + 
    ylim(0,1)+ 
    ggtitle(label = "Edge density across time",
            subtitle = paste(site,", Station", thestation))
  return(plot)
}

# F.Importing from excel file ----

xlsx_to_dataframe <- function(filename, method = "farm_A_15rows"){
  if(method == "farm_A_11rows"){
    df <- read_excel(filename,
                     col_types = c("date", "text", "skip",
                     "numeric", "numeric", "numeric",
                     "numeric", "numeric", "text", "text",
                     "text"))
    
  }
  if(method == "farm_A_15rows"){
    df <- read_excel(filename, 
                     col_types = c("date", "text", "numeric", 
                                   "numeric", "numeric", "numeric", 
                                   "numeric", "date", "date", "date", 
                                   "numeric", "numeric", "text", "text", 
                                   "date"))
  }
    return(df)
}

# New version of harmonize 2022-07-12
# handles hours in and out and strings first, then converts to posix
# catches station or Station
harmonize_feeder_data <- function(df, 
                                  groupstations = T, 
                                  method = "farm_A_clean",
                                  remove_filling = T,
                                  remove_na = T) {
  
  if(method == "farm_A_clean"){
    if(names(df)[3] == "Station"){
      names(df)[3] = "station"
    }
    
    dataset_raw <- df %>%
      mutate(
        id = as.factor(Tatouage),
        station = as.factor(station), # assumes station is column 4
        
        tempdate = as.character(as.Date(Date, origin = "1899-12-30")),
        
        hour.in = paste(tempdate,as.character(hms::as_hms(hredeb))) %>%
          as.POSIXct(),
        
        hour.out = paste(tempdate,as.character(hms::as_hms(hrefin)))%>%
          as.POSIXct(),
        
        visit_dur_secs = duree1 %>% 
          substr(12, 19) %>% 
          as.difftime()) %>%
      select(Date, id, station, pdsdeb, pdsfin, cons, hour.in, hour.out, visit_dur_secs) 
  }
  
  if(method == "farm_A_raw"){
    
    # catches station in uppercase
    if(names(df)[3] == "Station"){
      names(df)[3] = "station"
    }
    
    dataset_raw <- df %>%
      mutate(
        id = as.factor(Tatouage),
        station = as.factor(station), 
        
        tempdate = as.character(as.Date(Date, origin = "1899-12-30")),
        
        hour.in = hredeb %>%
          paste(tempdate,.) %>% 
          as.POSIXct(tz = ""),
        
        hour.out = hrefin %>%
          paste(tempdate,.) %>% 
          as.POSIXct(tz = "")
      )
    
    dataset_raw <- dataset_raw %>%
      mutate(visit_dur_secs = as.difftime(hour.out - hour.in)) %>%
      select(Date, id, station, pdsdeb, pdsfin, cons, hour.in, hour.out, visit_dur_secs) 
    
  }
  
  if(method == "deschambault"){
    dataset_raw <- df %>%
      mutate(
        Date = Date_fin,
        id = as.factor(Animal),
        station = as.factor(Parc),
        pdsdeb = Poids_aliment_debut,
        pdsfin = Poids_aliment_fin,
        cons = Qte_aliment,
        
        tempdate = as.character(as.Date(Date_fin)),
        
        hour.in = as.character(chron::times(Tdebut)),
        hour.in = as.POSIXct(paste(tempdate,hour.in), tz=""),
        
        hour.out = as.character(chron::times(Tfin)),
        hour.out = as.POSIXct(paste(tempdate,hour.out), tz=""),
        
        visit_dur_secs = as.difftime(Duree_s, units = "secs")
      ) %>%
      select(Date, id, station, pdsdeb, pdsfin, cons, hour.in, hour.out, visit_dur_secs) 
  }
  
  if(remove_filling == T){
    dataset_raw <- filter(dataset_raw, id != "FILLING") 
  }
  if(remove_na == T){
    dataset_raw <- drop_na(dataset_raw)   
  }
  
  if(groupstations == T){
    dataset_raw %>% # dropping NA values
      group_by(station) # test 2022-02-26 
  }
  
  return(dataset_raw)
  
} # end harmonize()


# old version of harmonize. does not work well with dates

harmonize_feeder_data_ <- function(df, 
                        groupstations = T, 
                        method = "farm_A_clean",
                        remove_filling = T,
                        remove_na = T) {
  
  if(method == "farm_A_clean"){
    dataset_raw <- df %>%
      mutate(
        id = as.factor(Tatouage),
        station = as.factor(station),
        
        tempdate = as.character(as.Date(Date, origin = "1899-12-30")),
        
        hour.in = as.character(chron::times(hredeb)),
        hour.in = as.POSIXct(paste(tempdate,hour.in), tz=""),
        
        hour.out = as.character(chron::times(hrefin)),
        hour.out = as.POSIXct(paste(tempdate,hour.out), tz=""),
        
        visit_dur_secs = as.difftime(duree1)) %>%
      select(Date, id, station, pdsdeb, pdsfin, cons, hour.in, hour.out, visit_dur_secs) 
    
    
    
  }
  
  if(method == "farm_A_raw"){
    dataset_raw <- df %>%
      mutate(
        id = as.factor(Tatouage),
        station = as.factor(station),
        
        tempdate = as.character(as.Date(Date)),
        
        hour.in = as.character(chron::times(hredeb)),
        hour.in = as.POSIXct(paste(tempdate,hour.in), tz=""),
        
        hour.out = as.character(chron::times(hrefin)),
        hour.out = as.POSIXct(paste(tempdate,hour.out), tz=""))
    
    dataset_raw <- dataset_raw %>%
      mutate(visit_dur_secs = as.difftime(hour.out - hour.in)) %>%
      select(Date, id, station, pdsdeb, pdsfin, cons, hour.in, hour.out, visit_dur_secs) 

  }
  
  if(method == "deschambault"){
    dataset_raw <- df %>%
      mutate(
        Date = Date_fin,
        id = as.factor(Animal),
        station = as.factor(Parc),
        pdsdeb = Poids_aliment_debut,
        pdsfin = Poids_aliment_fin,
        cons = Qte_aliment,
        
        tempdate = as.character(as.Date(Date_fin)),
        
        hour.in = as.character(chron::times(Tdebut)),
        hour.in = as.POSIXct(paste(tempdate,hour.in), tz=""),
        
        hour.out = as.character(chron::times(Tfin)),
        hour.out = as.POSIXct(paste(tempdate,hour.out), tz=""),
        
        visit_dur_secs = as.difftime(Duree_s, units = "secs")
      ) %>%
      select(Date, id, station, pdsdeb, pdsfin, cons, hour.in, hour.out, visit_dur_secs) 
  }
  
  if(remove_filling == T){
    dataset_raw <- filter(dataset_raw, id != "FILLING") 
  }
  if(remove_na == T){
    dataset_raw <- drop_na(dataset_raw)   
  }
  
  if(groupstations == T){
    dataset_raw %>% # dropping NA values
      group_by(station) # test 2022-02-26 
  }
  
  return(dataset_raw)
  
} # end harmonize()

# deprecated
standardize <- function(df, 
         groupstations = T, 
         method = "farm_A_clean",
         remove_filling = T,
         remove_na = T) {
  
  if(method == "farm_A_clean"){
    dataset_raw <- df %>%
      mutate(
        id = as.factor(Tatouage),
        station = as.factor(Station),
        
        tempdate = as.character(as.Date(Date)),
        
        hour.in = as.character(chron::times(hredeb)),
        hour.in = as.POSIXct(paste(tempdate,hour.in), tz=""),
        
        hour.out = as.character(chron::times(hrefin)),
        hour.out = as.POSIXct(paste(tempdate,hour.out), tz=""),
        
        visit_dur_secs = as.difftime(duree1)) %>%
      select(Date, id, station, pdsdeb, pdsfin, cons, hour.in, hour.out, visit_dur_secs) 
    
    
    
  }
  
  if(method == "farm_A_raw"){
    dataset_raw <- df %>%
      mutate(
        id = as.factor(Tatouage),
        station = as.factor(station),
        
        tempdate = as.character(as.Date(Date)),
        
        hour.in = as.character(chron::times(hredeb)),
        hour.in = as.POSIXct(paste(tempdate,hour.in), tz=""),
        
        hour.out = as.character(chron::times(hrefin)),
        hour.out = as.POSIXct(paste(tempdate,hour.out), tz=""))
    
    dataset_raw <- dataset_raw %>%
      mutate(visit_dur_secs = as.difftime(hour.out - hour.in)) %>%
      select(Date, id, station, pdsdeb, pdsfin, cons, hour.in, hour.out, visit_dur_secs) 
    
  }
  
  if(method == "deschambault"){
    dataset_raw <- df %>%
      mutate(
        Date = Date_fin,
        id = as.factor(Animal),
        station = as.factor(Parc),
        pdsdeb = Poids_aliment_debut,
        pdsfin = Poids_aliment_fin,
        cons = Qte_aliment,
        
        tempdate = as.character(as.Date(Date_fin)),
        
        hour.in = as.character(chron::times(Tdebut)),
        hour.in = as.POSIXct(paste(tempdate,hour.in), tz=""),
        
        hour.out = as.character(chron::times(Tfin)),
        hour.out = as.POSIXct(paste(tempdate,hour.out), tz=""),
        
        visit_dur_secs = as.difftime(Duree_s, units = "secs")
      ) %>%
      select(Date, id, station, pdsdeb, pdsfin, cons, hour.in, hour.out, visit_dur_secs) 
  }
  
  if(remove_filling == T){
    dataset_raw <- filter(dataset_raw, id != "FILLING") 
  }
  if(remove_na == T){
    dataset_raw <- drop_na(dataset_raw)   
  }
  
  if(groupstations == T){
    dataset_raw %>% # dropping NA values
      group_by(station) # test 2022-02-26 
  }
  
  return(dataset_raw)
  
}


# Same as previous but gives a nice dataframe ----
makeListPerStation <- function(df, thestation, doMerge = T) {
  df_s_n <- df %>%
    filter(station == thestation) %>%
    arrange(Date, hour.in)
  
  dataset_perday <- df_s_n %>% 
    ungroup() %>% # remove station grouping
    group_by(Date) %>% 
    mutate(date_ymd = Date) %>%
    group_map(~ mutate(.x, 
                       gap_duration = calcGapDuration(hour.in, hour.out))) %>%
    setNames(unique(sort(df_s_n$Date)))# this line preserves station number!
  
  # if doMerge is true, then successive visits for the same pig are merged. 
  if(doMerge == "T"){
    dataset_perday.merged <- lapply(dataset_perday, function(i) mergevisits(i)) 
    
    dataset_perday.merged.gd <- gapdurationlist(dataset_perday.merged)
    
    return(dataset_perday.merged.gd)
  } else {
    dataset_perday$id <- as.character(dataset_perday$id)
    return(dataset_perday)
  }
  
}

makeAllStationsPerdate <- function(df, domerge){
  output <- list()
  stations <- as.numeric(as.character(unique(df$station)))
  for (i in seq_along(stations)) {
    #message(c("Creating item ", i))
    output[[i]] <- makeListPerStation(df, stations[i], doMerge = domerge)
  }
  names(output) <- stations
  return(output)
}



mergevisits <- function(df) {
  output <- df %>%
    filter(!is.na(gap_duration)) %>%
    arrange(hour.in) %>%
    mutate(id2 = rleid(id)) %>%
    group_by(id2) %>%
    summarise(id = first(id), 
              hour.in = first(hour.in), 
              hour.out = last(hour.out),
              myDate = min(date_ymd)
    )%>%
    ungroup() %>%
    select(id, myDate, hour.in, hour.out)
  output$gap_duration <- calcGapDuration(output$hour.in, output$hour.out)
  return(output)
}



# Variant that corrects per feeding bout ----

getmetheplot_pliz_but_corrected_this_time <- function(site, df, thestation) {
  df_s_n <- df[df$station == thestation,] %>%
    arrange(Date, hour.in)
  dataset_perday <- df_s_n %>% 
    ungroup() %>% # remove station grouping
    group_by(Date) %>% 
    mutate(date_ymd = Date) %>%
    group_map(~ mutate(.x, 
                       gap_duration = calcGapDuration(hour.in, hour.out))) %>%
    setNames(unique(sort(df_s_n$Date))) # this line preserves station number!
  
  alldates <- names(dataset_perday)
  
  # Getting bouts per day to correct it later
  boutsperday <- sapply(dataset_perday, function(i) nrow(i))
  
  dataset_perday.merged <- lapply(dataset_perday, function(i) mergevisits(i)) 
  dataset_perday.merged.gd <- gapdurationlist(dataset_perday.merged)
  
  pairvisits_list <- get_pairs_list(dataset_perday.merged.gd, mythreshold = 20)
  graph_list <- lapply(pairvisits_list, function(i) graph_by_list_element(i,T))
  ggg_simple <- lapply(graph_list, function(i) simplify(i))
  ggg_simple <- createdegreeslist(ggg_simple)
  
  ed_bydate <- sapply(ggg_simple, function(i) edge_density(i)) %>% 
    rbind(names(dataset_perday)) %>% 
    as.data.frame() %>%
    transpose() %>%
    rename(date = V2,
           edgeDensity = V1) %>%
    mutate(corrected_ed = as.numeric(edgeDensity) / boutsperday)
  
  plot <- ggplot(ed_bydate, aes(x = as.Date(date), y = as.numeric(corrected_ed))) + geom_point() + 
    scale_x_date(date_breaks = "2 weeks") + 
    ylab("Edge density/bouts per day") + 
    xlab("Date") + 
    #ylim(0,1)+ 
    ggtitle(label = "Edge density across time",
            subtitle = paste("Corrected daily amount of feeding bouts.",site,", Station", thestation))
  return(plot)
}

# used in progressreportmarch ----

# DEPRECATED: it did not handle very well the lists, so I remade it in the function
# below

# populationPlot <- function(df_list){
#   k <- data.frame()
#   
#   for (station_n in seq_along(df_list)) {
#     f <- sapply(df_list[[station_n]], function(i) summarize(i, n = length(unique(id)))) %>%
#       collapse::unlist2d() %>%
#       reshape2::melt() 
#     
#     l <- as.data.frame(cbind(thedate = names(df_list[[station_n]]), n = f$value)) 
#     l$station_n <- names(df_list)[station_n]
#     k <- rbind(k,l)
#   }
#   
#   
#   k$thedate <- as.Date(k$thedate)
#   k$n <- as.numeric(k$n)
#   k$station_n <- as.factor(k$station_n)
#   
#   
#   ggplot(k, aes(x = thedate, y = n)) + geom_point(stat = "identity") + 
#     scale_x_date(date_breaks = "1 month") + facet_wrap(~station_n)
#   
# }

populationPlot <- function(df_raw) {
  
  populationdaily <- df_raw %>%
    ungroup() %>%
    filter(id != "FILLING") %>%
    group_by(station, Date) %>%
    summarise(n = length(unique(id))) 
  
  populationdaily$Date <- as.Date(populationdaily$Date)
  # fills missing dates by 0
  populationdaily <- populationdaily %>% pad %>% fill_by_value(value = 0)
  
  ggplot(populationdaily, aes(x = Date, y = n)) + geom_line(stat = "identity", color ="red") + 
    scale_x_date(date_breaks = "1 month") + facet_wrap(~station) + 
    theme(axis.text.x = element_text(angle = 90, 
                                     vjust = 0.5, 
                                     hjust = 1)) + 
    ylim(0, max(populationdaily$n))
  
}

plotConsData <- function(df, method ,thefarm) {
  if(method == "group"){
    theplot <- ggplot(data = SCH_cons_ps, aes(x = as.Date(Date))) + 
      geom_line(colour = "red", aes(y = daily.cons.group)) +
      scale_x_date(date_breaks = "1 month") + 
      facet_wrap(~station) + 
      theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1)) + 
      ylab("total daily consumption (kg)") +
      ggtitle(paste0("Whole group daily total consumption, ", thefarm," farm"))
    return(theplot)
  }
  if(method == "mean"){
    theplot <- ggplot(data = SCH_cons_ps, aes(x = as.Date(Date))) + 
      geom_line(colour = "red", aes(y = mean.daily.cons)) +
      scale_x_date(date_breaks = "1 month") + 
      facet_wrap(~station) + 
      theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1)) + 
      ylab("mean daily consumption/pig (kg)") +
      ggtitle(paste0("Mean group consumption, ", thefarm," farm"))
    return(theplot)
  }
}
